@@ -19,6 +19,7 @@ from oslo_utils import uuidutils
 
 from octavia.common import constants
 import octavia.common.context
+from octavia.common import exceptions
 from octavia.db import repositories
 from octavia.tests.functional.api.v2 import base
 
@@ -103,20 +104,16 @@ class TestL7Rule(base.BaseAPITest):
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(self.NOT_AUTHORIZED_BODY, response)
 
-    def test_get_hides_deleted(self):
+    def test_get_deleted_gives_404(self):
         api_l7rule = self.create_l7rule(
             self.l7policy_id, constants.L7RULE_TYPE_PATH,
             constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
             '/api').get(self.root_tag)
 
-        response = self.get(self.l7rules_path)
-        objects = response.json.get(self.root_tag_list)
-        self.assertEqual(len(objects), 1)
         self.set_object_status(self.l7rule_repo, api_l7rule.get('id'),
                                provisioning_status=constants.DELETED)
-        response = self.get(self.l7rules_path)
-        objects = response.json.get(self.root_tag_list)
-        self.assertEqual(len(objects), 0)
+        self.get(self.l7rule_path.format(l7rule_id=api_l7rule.get('id')),
+                 status=404)
 
     def test_get_bad_parent_policy(self):
         bad_path = (self.L7RULES_PATH.format(
@@ -360,6 +357,21 @@ class TestL7Rule(base.BaseAPITest):
         self.assertIsInstance(response, list)
         self.assertEqual(0, len(response))
 
+    def test_get_all_hides_deleted(self):
+        api_l7rule = self.create_l7rule(
+            self.l7policy_id, constants.L7RULE_TYPE_PATH,
+            constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
+            '/api').get(self.root_tag)
+
+        response = self.get(self.l7rules_path)
+        objects = response.json.get(self.root_tag_list)
+        self.assertEqual(len(objects), 1)
+        self.set_object_status(self.l7rule_repo, api_l7rule.get('id'),
+                               provisioning_status=constants.DELETED)
+        response = self.get(self.l7rules_path)
+        objects = response.json.get(self.root_tag_list)
+        self.assertEqual(len(objects), 0)
+
     def test_create_host_name_rule(self):
         api_l7rule = self.create_l7rule(
             self.l7policy_id, constants.L7RULE_TYPE_HOST_NAME,
@@ -534,6 +546,7 @@ class TestL7Rule(base.BaseAPITest):
             l7rule_prov_status=constants.PENDING_CREATE,
             l7rule_op_status=constants.OFFLINE)
 
+    @mock.patch('octavia.common.constants.MAX_L7RULES_PER_L7POLICY', new=2)
     def test_create_too_many_rules(self):
         for i in range(0, constants.MAX_L7RULES_PER_L7POLICY):
             self.create_l7rule(
@@ -580,17 +593,19 @@ class TestL7Rule(base.BaseAPITest):
                   'value': 'some-string'}
         self.post(self.l7rules_path, self._build_body(l7rule), status=400)
 
-    def test_create_with_bad_handler(self):
-        self.handler_mock().l7rule.create.side_effect = Exception()
-        api_l7rule = self.create_l7rule(
-            self.l7policy_id, constants.L7RULE_TYPE_PATH,
-            constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
-            '/api').get(self.root_tag)
-        self.assert_correct_status(
-            lb_id=self.lb_id, listener_id=self.listener_id,
-            l7policy_id=self.l7policy_id, l7rule_id=api_l7rule.get('id'),
-            l7rule_prov_status=constants.ERROR,
-            l7rule_op_status=constants.OFFLINE)
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_create_with_bad_provider(self, mock_provider):
+        mock_provider.side_effect = exceptions.ProviderDriverError(
+            prov='bad_driver', user_msg='broken')
+        l7rule = {'compare_type': 'REGEX',
+                  'invert': False,
+                  'type': 'PATH',
+                  'value': '/images*',
+                  'admin_state_up': True}
+        response = self.post(self.l7rules_path, self._build_body(l7rule),
+                             status=500)
+        self.assertIn('Provider \'bad_driver\' reports error: broken',
+                      response.json.get('faultstring'))
 
     def test_update(self):
         api_l7rule = self.create_l7rule(
@@ -602,7 +617,7 @@ class TestL7Rule(base.BaseAPITest):
         response = self.put(self.l7rule_path.format(
             l7rule_id=api_l7rule.get('id')),
             self._build_body(new_l7rule)).json.get(self.root_tag)
-        self.assertEqual('/api', response.get('value'))
+        self.assertEqual('/images', response.get('value'))
         self.assert_correct_status(
             lb_id=self.lb_id, listener_id=self.listener_id,
             l7policy_id=self.l7policy_id, l7rule_id=api_l7rule.get('id'),
@@ -644,7 +659,7 @@ class TestL7Rule(base.BaseAPITest):
                     l7rule_id=api_l7rule.get('id')),
                     self._build_body(new_l7rule)).json.get(self.root_tag)
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
-        self.assertEqual('/api', response.get('value'))
+        self.assertEqual('/images', response.get('value'))
         self.assert_correct_status(
             lb_id=self.lb_id, listener_id=self.listener_id,
             l7policy_id=self.l7policy_id, l7rule_id=api_l7rule.get('id'),
@@ -688,20 +703,21 @@ class TestL7Rule(base.BaseAPITest):
         self.put(self.l7rule_path.format(l7rule_id=l7rule.get('id')),
                  self._build_body(new_l7rule), status=400)
 
-    def test_update_with_bad_handler(self):
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_update_with_bad_provider(self, mock_provider):
         api_l7rule = self.create_l7rule(
             self.l7policy_id, constants.L7RULE_TYPE_PATH,
             constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
             '/api').get(self.root_tag)
         self.set_lb_status(self.lb_id)
         new_l7rule = {'value': '/images'}
-        self.handler_mock().l7rule.update.side_effect = Exception()
-        self.put(self.l7rule_path.format(
-            l7rule_id=api_l7rule.get('id')), self._build_body(new_l7rule))
-        self.assert_correct_status(
-            lb_id=self.lb_id, listener_id=self.listener_id,
-            l7policy_id=self.l7policy_id, l7rule_id=api_l7rule.get('id'),
-            l7rule_prov_status=constants.ERROR)
+        mock_provider.side_effect = exceptions.ProviderDriverError(
+            prov='bad_driver', user_msg='broken')
+        response = self.put(
+            self.l7rule_path.format(l7rule_id=api_l7rule.get('id')),
+            self._build_body(new_l7rule), status=500)
+        self.assertIn('Provider \'bad_driver\' reports error: broken',
+                      response.json.get('faultstring'))
 
     def test_update_with_invalid_rule(self):
         api_l7rule = self.create_l7rule(
@@ -832,7 +848,8 @@ class TestL7Rule(base.BaseAPITest):
         self.delete(self.l7rule_path.format(
             l7rule_id=uuidutils.generate_uuid()), status=404)
 
-    def test_delete_with_bad_handler(self):
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_delete_with_bad_provider(self, mock_provider):
         api_l7rule = self.create_l7rule(
             self.l7policy_id, constants.L7RULE_TYPE_PATH,
             constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
@@ -848,12 +865,10 @@ class TestL7Rule(base.BaseAPITest):
         self.assertIsNotNone(response.pop('updated_at'))
         self.assertEqual(api_l7rule, response)
 
-        self.handler_mock().l7rule.delete.side_effect = Exception()
-        self.delete(self.l7rule_path.format(l7rule_id=api_l7rule.get('id')))
-        self.assert_correct_status(
-            lb_id=self.lb_id, listener_id=self.listener_id,
-            l7policy_id=self.l7policy_id, l7rule_id=api_l7rule.get('id'),
-            l7rule_prov_status=constants.ERROR)
+        mock_provider.side_effect = exceptions.ProviderDriverError(
+            prov='bad_driver', user_msg='broken')
+        self.delete(self.l7rule_path.format(l7rule_id=api_l7rule.get('id')),
+                    status=500)
 
     def test_create_when_lb_pending_update(self):
         self.create_l7rule(
@@ -937,6 +952,21 @@ class TestL7Rule(base.BaseAPITest):
         self.delete(self.l7rule_path.format(l7rule_id=l7rule.get('id')),
                     status=409)
 
+    def test_update_already_deleted(self):
+        l7rule = self.create_l7rule(
+            self.l7policy_id, constants.L7RULE_TYPE_PATH,
+            constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
+            '/api').get(self.root_tag)
+        # This updates the child objects
+        self.set_lb_status(self.lb_id, status=constants.DELETED)
+        new_l7rule = {'type': constants.L7RULE_TYPE_COOKIE,
+                      'compare_type':
+                          constants.L7RULE_COMPARE_TYPE_ENDS_WITH,
+                      'value': 'some-string',
+                      'key': 'some-cookie'}
+        self.put(self.l7rule_path.format(l7rule_id=l7rule.get('id')),
+                 body=self._build_body(new_l7rule), status=404)
+
     def test_delete_already_deleted(self):
         l7rule = self.create_l7rule(
             self.l7policy_id, constants.L7RULE_TYPE_PATH,
@@ -945,4 +975,4 @@ class TestL7Rule(base.BaseAPITest):
         # This updates the child objects
         self.set_lb_status(self.lb_id, status=constants.DELETED)
         self.delete(self.l7rule_path.format(l7rule_id=l7rule.get('id')),
-                    status=204)
+                    status=404)
