@@ -19,6 +19,7 @@ from oslo_config import fixture as oslo_fixture
 from taskflow.patterns import linear_flow as flow
 
 from octavia.common import constants
+from octavia.common import data_models
 from octavia.controller.worker.flows import amphora_flows
 import octavia.tests.unit.base as base
 
@@ -32,17 +33,17 @@ class TestAmphoraFlows(base.TestCase):
 
     def setUp(self):
         super(TestAmphoraFlows, self).setUp()
-        old_amp_driver = cfg.CONF.controller_worker.amphora_driver
-        cfg.CONF.set_override('amphora_driver', 'amphora_haproxy_rest_driver',
-                              group='controller_worker')
-        cfg.CONF.set_override('enable_anti_affinity', False,
-                              group='nova')
+        self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
+        self.conf.config(
+            group="controller_worker",
+            amphora_driver='amphora_haproxy_rest_driver')
+        self.conf.config(group="nova", enable_anti_affinity=False)
         self.AmpFlow = amphora_flows.AmphoraFlows()
-        conf = oslo_fixture.Config(cfg.CONF)
-        conf.config(group="keystone_authtoken", auth_version=AUTH_VERSION)
-
-        self.addCleanup(cfg.CONF.set_override, 'amphora_driver',
-                        old_amp_driver, group='controller_worker')
+        self.amp1 = data_models.Amphora(id=1)
+        self.amp2 = data_models.Amphora(id=2)
+        self.amp3 = data_models.Amphora(id=3, status=constants.DELETED)
+        self.lb = data_models.LoadBalancer(
+            id=4, amphorae=[self.amp1, self.amp2, self.amp3])
 
     def test_get_create_amphora_flow(self, mock_get_net_driver):
 
@@ -56,7 +57,7 @@ class TestAmphoraFlows(base.TestCase):
         self.assertIn(constants.SERVER_PEM, amp_flow.provides)
 
         self.assertEqual(5, len(amp_flow.provides))
-        self.assertEqual(0, len(amp_flow.requires))
+        self.assertEqual(1, len(amp_flow.requires))
 
     def test_get_create_amphora_flow_cert(self, mock_get_net_driver):
         self.AmpFlow = amphora_flows.AmphoraFlows()
@@ -70,7 +71,7 @@ class TestAmphoraFlows(base.TestCase):
         self.assertIn(constants.COMPUTE_ID, amp_flow.provides)
 
         self.assertEqual(5, len(amp_flow.provides))
-        self.assertEqual(0, len(amp_flow.requires))
+        self.assertEqual(1, len(amp_flow.requires))
 
     def test_get_create_amphora_for_lb_flow(self, mock_get_net_driver):
 
@@ -88,11 +89,9 @@ class TestAmphoraFlows(base.TestCase):
         self.assertIn(constants.SERVER_PEM, amp_flow.provides)
 
         self.assertEqual(5, len(amp_flow.provides))
-        self.assertEqual(1, len(amp_flow.requires))
+        self.assertEqual(2, len(amp_flow.requires))
 
     def test_get_cert_create_amphora_for_lb_flow(self, mock_get_net_driver):
-        cfg.CONF.set_override('amphora_driver', 'amphora_haproxy_rest_driver',
-                              group='controller_worker')
 
         self.AmpFlow = amphora_flows.AmphoraFlows()
 
@@ -110,12 +109,10 @@ class TestAmphoraFlows(base.TestCase):
         self.assertIn(constants.SERVER_PEM, amp_flow.provides)
 
         self.assertEqual(5, len(amp_flow.provides))
-        self.assertEqual(1, len(amp_flow.requires))
+        self.assertEqual(2, len(amp_flow.requires))
 
     def test_get_cert_master_create_amphora_for_lb_flow(
             self, mock_get_net_driver):
-        cfg.CONF.set_override('amphora_driver', 'amphora_haproxy_rest_driver',
-                              group='controller_worker')
 
         self.AmpFlow = amphora_flows.AmphoraFlows()
 
@@ -133,15 +130,12 @@ class TestAmphoraFlows(base.TestCase):
         self.assertIn(constants.SERVER_PEM, amp_flow.provides)
 
         self.assertEqual(5, len(amp_flow.provides))
-        self.assertEqual(1, len(amp_flow.requires))
+        self.assertEqual(2, len(amp_flow.requires))
 
     def test_get_cert_master_rest_anti_affinity_create_amphora_for_lb_flow(
             self, mock_get_net_driver):
-        cfg.CONF.set_override('amphora_driver', 'amphora_haproxy_rest_driver',
-                              group='controller_worker')
 
-        cfg.CONF.set_override('enable_anti_affinity', True,
-                              group='nova')
+        self.conf.config(group="nova", enable_anti_affinity=True)
 
         self.AmpFlow = amphora_flows.AmphoraFlows()
         amp_flow = self.AmpFlow._get_create_amp_for_lb_subflow(
@@ -155,7 +149,8 @@ class TestAmphoraFlows(base.TestCase):
         self.assertIn(constants.SERVER_PEM, amp_flow.provides)
 
         self.assertEqual(5, len(amp_flow.provides))
-        self.assertEqual(2, len(amp_flow.requires))
+        self.assertEqual(3, len(amp_flow.requires))
+        self.conf.config(group="nova", enable_anti_affinity=False)
 
     def test_get_cert_backup_create_amphora_for_lb_flow(
             self, mock_get_net_driver):
@@ -175,7 +170,7 @@ class TestAmphoraFlows(base.TestCase):
         self.assertIn(constants.SERVER_PEM, amp_flow.provides)
 
         self.assertEqual(5, len(amp_flow.provides))
-        self.assertEqual(1, len(amp_flow.requires))
+        self.assertEqual(2, len(amp_flow.requires))
 
     def test_get_cert_bogus_create_amphora_for_lb_flow(
             self, mock_get_net_driver):
@@ -195,14 +190,11 @@ class TestAmphoraFlows(base.TestCase):
         self.assertIn(constants.SERVER_PEM, amp_flow.provides)
 
         self.assertEqual(5, len(amp_flow.provides))
-        self.assertEqual(1, len(amp_flow.requires))
+        self.assertEqual(2, len(amp_flow.requires))
 
     def test_get_cert_backup_rest_anti_affinity_create_amphora_for_lb_flow(
             self, mock_get_net_driver):
-        cfg.CONF.set_override('amphora_driver', 'amphora_haproxy_rest_driver',
-                              group='controller_worker')
-        cfg.CONF.set_override('enable_anti_affinity', True,
-                              group='nova')
+        self.conf.config(group="nova", enable_anti_affinity=True)
 
         self.AmpFlow = amphora_flows.AmphoraFlows()
         amp_flow = self.AmpFlow._get_create_amp_for_lb_subflow(
@@ -216,7 +208,8 @@ class TestAmphoraFlows(base.TestCase):
         self.assertIn(constants.SERVER_PEM, amp_flow.provides)
 
         self.assertEqual(5, len(amp_flow.provides))
-        self.assertEqual(2, len(amp_flow.requires))
+        self.assertEqual(3, len(amp_flow.requires))
+        self.conf.config(group="nova", enable_anti_affinity=False)
 
     def test_get_delete_amphora_flow(self, mock_get_net_driver):
 
@@ -247,83 +240,98 @@ class TestAmphoraFlows(base.TestCase):
         result = self.AmpFlow._create_new_amp_for_lb_decider(history)
         self.assertFalse(result)
 
-    def test_get_failover_flow(self, mock_get_net_driver):
+    def test_get_failover_flow_allocated(self, mock_get_net_driver):
+
+        amp_flow = self.AmpFlow.get_failover_flow(
+            load_balancer=self.lb)
+
+        self.assertIsInstance(amp_flow, flow.Flow)
+
+        self.assertIn(constants.FAILED_AMPHORA, amp_flow.requires)
+        self.assertIn(constants.LOADBALANCER_ID, amp_flow.requires)
+
+        self.assertIn(constants.AMP_DATA, amp_flow.provides)
+        self.assertIn(constants.AMPHORA, amp_flow.provides)
+        self.assertIn(constants.AMPHORA_ID, amp_flow.provides)
+        self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, amp_flow.provides)
+        self.assertIn(constants.COMPUTE_ID, amp_flow.provides)
+        self.assertIn(constants.COMPUTE_OBJ, amp_flow.provides)
+        self.assertIn(constants.LISTENERS, amp_flow.provides)
+        self.assertIn(constants.LOADBALANCER, amp_flow.provides)
+
+        self.assertEqual(3, len(amp_flow.requires))
+        self.assertEqual(12, len(amp_flow.provides))
+
+        amp_flow = self.AmpFlow.get_failover_flow(
+            role=constants.ROLE_MASTER, load_balancer=self.lb)
+
+        self.assertIsInstance(amp_flow, flow.Flow)
+
+        self.assertIn(constants.FAILED_AMPHORA, amp_flow.requires)
+        self.assertIn(constants.LOADBALANCER_ID, amp_flow.requires)
+
+        self.assertIn(constants.AMP_DATA, amp_flow.provides)
+        self.assertIn(constants.AMPHORA, amp_flow.provides)
+        self.assertIn(constants.AMPHORA_ID, amp_flow.provides)
+        self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, amp_flow.provides)
+        self.assertIn(constants.COMPUTE_ID, amp_flow.provides)
+        self.assertIn(constants.COMPUTE_OBJ, amp_flow.provides)
+        self.assertIn(constants.LISTENERS, amp_flow.provides)
+        self.assertIn(constants.LOADBALANCER, amp_flow.provides)
+
+        self.assertEqual(3, len(amp_flow.requires))
+        self.assertEqual(12, len(amp_flow.provides))
+
+        amp_flow = self.AmpFlow.get_failover_flow(
+            role=constants.ROLE_BACKUP, load_balancer=self.lb)
+
+        self.assertIsInstance(amp_flow, flow.Flow)
+
+        self.assertIn(constants.FAILED_AMPHORA, amp_flow.requires)
+        self.assertIn(constants.LOADBALANCER_ID, amp_flow.requires)
+
+        self.assertIn(constants.AMP_DATA, amp_flow.provides)
+        self.assertIn(constants.AMPHORA, amp_flow.provides)
+        self.assertIn(constants.AMPHORA_ID, amp_flow.provides)
+        self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, amp_flow.provides)
+        self.assertIn(constants.COMPUTE_ID, amp_flow.provides)
+        self.assertIn(constants.COMPUTE_OBJ, amp_flow.provides)
+        self.assertIn(constants.LISTENERS, amp_flow.provides)
+        self.assertIn(constants.LOADBALANCER, amp_flow.provides)
+
+        self.assertEqual(3, len(amp_flow.requires))
+        self.assertEqual(12, len(amp_flow.provides))
+
+        amp_flow = self.AmpFlow.get_failover_flow(
+            role='BOGUSROLE', load_balancer=self.lb)
+
+        self.assertIsInstance(amp_flow, flow.Flow)
+
+        self.assertIn(constants.FAILED_AMPHORA, amp_flow.requires)
+        self.assertIn(constants.LOADBALANCER_ID, amp_flow.requires)
+
+        self.assertIn(constants.AMP_DATA, amp_flow.provides)
+        self.assertIn(constants.AMPHORA, amp_flow.provides)
+        self.assertIn(constants.AMPHORA_ID, amp_flow.provides)
+        self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, amp_flow.provides)
+        self.assertIn(constants.COMPUTE_ID, amp_flow.provides)
+        self.assertIn(constants.COMPUTE_OBJ, amp_flow.provides)
+        self.assertIn(constants.LISTENERS, amp_flow.provides)
+        self.assertIn(constants.LOADBALANCER, amp_flow.provides)
+
+        self.assertEqual(3, len(amp_flow.requires))
+        self.assertEqual(12, len(amp_flow.provides))
+
+    def test_get_failover_flow_spare(self, mock_get_net_driver):
 
         amp_flow = self.AmpFlow.get_failover_flow()
 
         self.assertIsInstance(amp_flow, flow.Flow)
 
         self.assertIn(constants.FAILED_AMPHORA, amp_flow.requires)
-        self.assertIn(constants.LOADBALANCER_ID, amp_flow.requires)
 
-        self.assertIn(constants.AMP_DATA, amp_flow.provides)
-        self.assertIn(constants.AMPHORA, amp_flow.provides)
-        self.assertIn(constants.AMPHORA_ID, amp_flow.provides)
-        self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, amp_flow.provides)
-        self.assertIn(constants.COMPUTE_ID, amp_flow.provides)
-        self.assertIn(constants.COMPUTE_OBJ, amp_flow.provides)
-        self.assertIn(constants.LISTENERS, amp_flow.provides)
-        self.assertIn(constants.LOADBALANCER, amp_flow.provides)
-
-        self.assertEqual(2, len(amp_flow.requires))
-        self.assertEqual(11, len(amp_flow.provides))
-
-        amp_flow = self.AmpFlow.get_failover_flow(role=constants.ROLE_MASTER)
-
-        self.assertIsInstance(amp_flow, flow.Flow)
-
-        self.assertIn(constants.FAILED_AMPHORA, amp_flow.requires)
-        self.assertIn(constants.LOADBALANCER_ID, amp_flow.requires)
-
-        self.assertIn(constants.AMP_DATA, amp_flow.provides)
-        self.assertIn(constants.AMPHORA, amp_flow.provides)
-        self.assertIn(constants.AMPHORA_ID, amp_flow.provides)
-        self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, amp_flow.provides)
-        self.assertIn(constants.COMPUTE_ID, amp_flow.provides)
-        self.assertIn(constants.COMPUTE_OBJ, amp_flow.provides)
-        self.assertIn(constants.LISTENERS, amp_flow.provides)
-        self.assertIn(constants.LOADBALANCER, amp_flow.provides)
-
-        self.assertEqual(2, len(amp_flow.requires))
-        self.assertEqual(11, len(amp_flow.provides))
-
-        amp_flow = self.AmpFlow.get_failover_flow(role=constants.ROLE_BACKUP)
-
-        self.assertIsInstance(amp_flow, flow.Flow)
-
-        self.assertIn(constants.FAILED_AMPHORA, amp_flow.requires)
-        self.assertIn(constants.LOADBALANCER_ID, amp_flow.requires)
-
-        self.assertIn(constants.AMP_DATA, amp_flow.provides)
-        self.assertIn(constants.AMPHORA, amp_flow.provides)
-        self.assertIn(constants.AMPHORA_ID, amp_flow.provides)
-        self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, amp_flow.provides)
-        self.assertIn(constants.COMPUTE_ID, amp_flow.provides)
-        self.assertIn(constants.COMPUTE_OBJ, amp_flow.provides)
-        self.assertIn(constants.LISTENERS, amp_flow.provides)
-        self.assertIn(constants.LOADBALANCER, amp_flow.provides)
-
-        self.assertEqual(2, len(amp_flow.requires))
-        self.assertEqual(11, len(amp_flow.provides))
-
-        amp_flow = self.AmpFlow.get_failover_flow(role='BOGUSROLE')
-
-        self.assertIsInstance(amp_flow, flow.Flow)
-
-        self.assertIn(constants.FAILED_AMPHORA, amp_flow.requires)
-        self.assertIn(constants.LOADBALANCER_ID, amp_flow.requires)
-
-        self.assertIn(constants.AMP_DATA, amp_flow.provides)
-        self.assertIn(constants.AMPHORA, amp_flow.provides)
-        self.assertIn(constants.AMPHORA_ID, amp_flow.provides)
-        self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, amp_flow.provides)
-        self.assertIn(constants.COMPUTE_ID, amp_flow.provides)
-        self.assertIn(constants.COMPUTE_OBJ, amp_flow.provides)
-        self.assertIn(constants.LISTENERS, amp_flow.provides)
-        self.assertIn(constants.LOADBALANCER, amp_flow.provides)
-
-        self.assertEqual(2, len(amp_flow.requires))
-        self.assertEqual(11, len(amp_flow.provides))
+        self.assertEqual(1, len(amp_flow.requires))
+        self.assertEqual(0, len(amp_flow.provides))
 
     def test_cert_rotate_amphora_flow(self, mock_get_net_driver):
         self.AmpFlow = amphora_flows.AmphoraFlows()
@@ -347,7 +355,7 @@ class TestAmphoraFlows(base.TestCase):
         self.assertIn(constants.LOADBALANCER, vrrp_subflow.requires)
 
         self.assertEqual(1, len(vrrp_subflow.provides))
-        self.assertEqual(1, len(vrrp_subflow.requires))
+        self.assertEqual(2, len(vrrp_subflow.requires))
 
     def test_get_post_map_lb_subflow(self, mock_get_net_driver):
 
